@@ -34,6 +34,8 @@ import numpy
 import cPickle
 import zlib
 from factors import Factor, _find_levels
+import sys
+import ordereddict
 
 
 class DataFrame(object):
@@ -228,8 +230,13 @@ class DataFrame(object):
         old_name may also be the number of a column.
 
         """
+        if old_name == new_name:
+            return
         old_name = self._map_column_number_to_name(old_name)
-        self.value_dict[new_name] = self.value_dict[old_name]
+        try:
+            self.value_dict[new_name] = self.value_dict[old_name]
+        except KeyError:
+            raise KeyError("Column %s not found. Available were %s" % (old_name, self.columns_ordered))
         del self.value_dict[old_name]
         idx = self.columns_ordered.index(old_name)
         self.columns_ordered[idx] = new_name
@@ -265,7 +272,10 @@ class DataFrame(object):
                     if isinstance(value, str):
                         v_str = '"%s"' % value
                     else:
-                        v_str = str(value)
+                        try:
+                            v_str = str(value)
+                        except UnicodeEncodeError:
+                            v_str = value.encode('latin-1', 'replace')
                 else:
                     v_str = '--'
                 print >> buf, justify( v_str ),
@@ -456,6 +466,10 @@ class DataFrame(object):
             elif isinstance(selection[0], str) or isinstance(selection[0], numpy.str_):
                 rows_to_select = (self._map_row_name_to_number(selection[0]), )
             else:# neither a number/str, nor as slice request. this needs to be a sequence, either of int, then we can use it directly. or of boolean, which means we have to convert it into a list of row no.
+                #if selection[0] is False:
+                    #rows_to_select = numpy.zeros((self.num_rows,),dtype=numpy.bool)
+                #elif selection[0] is True:
+                    #rows_to_select = numpy.ones((self.num_rows,),dtype=numpy.bool)
                 if len(selection[0]) == 0:
                     rows_to_select = numpy.zeros((self.num_rows,),dtype=numpy.bool)
                 else:
@@ -541,7 +555,9 @@ class DataFrame(object):
         
         """
         columns_to_select, rows_to_select = self._interpret_selection(selection)
-        if len(columns_to_select) == 1 and len(rows_to_select) == 1:
+        if len(columns_to_select) == 1 and len(rows_to_select) == 1 and not (
+            isinstance(rows_to_select, numpy.ndarray) and rows_to_select.dtype == numpy.bool
+        ):
             if (isinstance(value, DataFrame) and 
                 value.num_rows == 1 and 
                 len(value.columns_ordered) == 1):
@@ -678,6 +694,15 @@ class DataFrame(object):
     def iter_rows(self):
         """Return an iterator over the rows (dicts).
 
+        """
+        def inner():
+            for i in xrange(0, self.num_rows):
+                yield self.get_row(i)
+        return IteratorPlusLength(inner(), self.num_rows)
+
+    def iter_rows_old(self):
+        """Return an iterator over the rows (dicts).
+
         """
         for i in xrange(0, self.num_rows):
             yield self.get_row(i)
@@ -897,6 +922,11 @@ class DataFrame(object):
 
         Direction (ascending) needs to be specified for each field.
         This is not an inplace sort, but returns a new DataFrame!
+
+        You can sort by one field: sort_by(my_field, ascending=False)
+        or by multiple fields: sort_by([my_fieldA, my_fieldB], [True, False]),
+        but then you will need to pass in the order for each one as well.
+
         
         """
         #input validation and converting into sequences
@@ -917,6 +947,9 @@ class DataFrame(object):
                              " as you have fields you're sorting on")
         field_or_fields = \
                 [self._map_column_number_to_name(x) for x in field_or_fields]
+        for f in field_or_fields:
+            if not f in self.columns_ordered:
+                raise KeyError("Unknown column_name: %s" % f)
         ziped = zip(field_or_fields, ascending)
         neworder = range(0, self.num_rows)
         #compare to values at a given index
@@ -975,6 +1008,12 @@ class DataFrame(object):
             return
         self.value_dict[column_name] = Factor(self.value_dict[column_name], levels)
 
+    def turn_into_character(self, column_name, levels = None):
+        """Convert a level column into it's character values"""
+        if not isinstance(self.value_dict[column_name], Factor):
+            return
+        self.value_dict[column_name] = self.value_dict[column_name].as_levels()
+
     def digitize_column(self, column_name, bins = None, no_of_bins = None, min=None, max = None):
         """Convert a column into a number of bin-ids"""
         if bins is None:
@@ -1047,7 +1086,7 @@ class DataFrame(object):
 
                     if self.value_dict[key].dtype.str.startswith('|O') or self.value_dict[key].dtype.str.startswith("|S"):
                         op_data.write('s')
-                        op_data.write(zlib.compress( cPickle.dumps(tuple([x for x in self.value_dict[key]]), cPickle.HIGHEST_PROTOCOL)))
+                        op_data.write(zlib.compress( cPickle.dumps(tuple([str(x) for x in self.value_dict[key]]), cPickle.HIGHEST_PROTOCOL)))
                     else:
                         op_data.write('n')
                         numpy.save(op_data, numpy.ma.getdata(self.value_dict[key]))
@@ -1074,7 +1113,14 @@ class DataFrame(object):
                     op_data, op_mask = StringIO.StringIO(state[k][key][0]),StringIO.StringIO(state[k][key][1])
                     data_type = op_data.read(1)
                     if data_type == 's':
-                        data = numpy.array(cPickle.loads(zlib.decompress(op_data.read())),dtype=numpy.object)
+                        comp = op_data.read()
+                        z = zlib.decompress(comp)
+                        try:
+                            tups = cPickle.loads(z)
+                            data = numpy.array(tups,dtype=numpy.object)
+                        except:
+                            import pdb
+                            pdb.post_mortem()
                     elif data_type == 'n':
                         data = numpy.load(op_data)
                     else: #we're abusing that pickled numpy data starts with \x93NUMPY to preserve backwards compability
@@ -1101,7 +1147,7 @@ class DataFrame(object):
         for o in (order, last_order):
             for column_name in o:
                 if not column_name in self.columns_ordered:
-                    raise KeyError("Column does not exist: %s" % column_name)
+                    raise KeyError("Column does not exist: %s" % (column_name,))
         remaining = []
         for column_name in self.columns_ordered:
             if not column_name in order and not column_name in last_order:
@@ -1113,6 +1159,8 @@ class DataFrame(object):
         self.columns_ordered = columns_ordered
 
     def melt(self, id_vars, measure_vars, measurement_name_target = 'Measurement', value_target = 'Value' ):
+        if isinstance(id_vars, str):
+            raise ValueError("id_vars needs to be alist, not a string")
         result = { measurement_name_target: [], value_target: []}
         for id_var in id_vars:
             result[id_var] = []
@@ -1128,8 +1176,31 @@ class DataFrame(object):
                 result.value_dict[id_var] = Factor(result.value_dict[id_var], self.value_dict[id_var].levels, False)
         return result
 
-
-
+    def sort_by_minimal_distance(self, columns_to_consider, distance_function = 'euclid'):
+        if distance_function == 'euclid':
+            def euclid(a, b):
+                sum = 0
+                for ii in xrange(0, len(a)):
+                    sum += (a[ii] - b[ii])**2
+                return sum
+            distance_function = euclid
+        final_order = []
+        temp_order = range(0, len(self))
+        final_order.append(temp_order.pop())
+        lookat = self[:, columns_to_consider]
+        while temp_order:
+            shortest_distance = sys.maxint
+            select = None
+            col_current = lookat[final_order[-1], :].as2DMatrix()
+            for name in temp_order:
+                col_test = lookat[name, :].as2DMatrix()
+                distance = distance_function(col_current, col_test)
+                if distance < shortest_distance:
+                    shortest_distance = distance
+                    select = name
+            final_order.append(select)
+            temp_order.remove(select)
+        return self[final_order, :]
 
 class _ShapeAttribute(object):
 
@@ -1171,8 +1242,12 @@ class _RowNamesAttribute(object):
         if not obj._row_names_ordered is None:
             lookup = {}
             for row_no, value in enumerate(obj._row_names_ordered):
-                if value in lookup:
-                    raise ValueError("Duplicate row name: %s" % value)
+                try:
+                    if value in lookup:
+                        raise ValueError("Duplicate row name: %s" % value)
+                except: #happens if 'value in lookup' fails (not hashable?)
+                    print 'issue is with %s' % value
+                    raise
                 lookup[value] = row_no
             obj._row_names_dict = lookup
 DataFrame.row_names = _RowNamesAttribute()
@@ -1235,8 +1310,13 @@ def _convert_to_numpy(seq, column_name):
     #any large uint -> large uint
     #any bool -> bolo
     #other -> object
-    if (str in types_present or unicode in types_present or numpy.string_ in types_present):
+    if (unicode in types_present):
+        seq = [s is None and "" or unicode(s) for s in seq]
+        max_length = max(len(unicode(s)) for s in seq if not s is None)
+        dtype = numpy.object
+    elif (str in types_present or numpy.string_ in types_present):
         max_length = max(len(str(s)) for s in seq if not s is None)
+        seq = [s is None and "" or str(s) for s in seq]
         dtype = numpy.object
     elif (numpy.float64 in types_present):
         dtype= numpy.float64
@@ -1248,7 +1328,14 @@ def _convert_to_numpy(seq, column_name):
         dtype = numpy.int64
     elif numpy.uint64 in types_present:
         dtype = numpy.uint64
-    elif int in types_present or numpy.int32 in types_present:
+    elif int in types_present:
+        max_int_value = 2147483647
+        dtype = numpy.int32
+        for value in seq:
+            if value > max_int_value:
+                dtype = numpy.int64
+                break
+    elif numpy.int32 in types_present:
         dtype = numpy.int32
     elif numpy.uint32 in types_present:
         dtype = numpy.uint32
@@ -1281,6 +1368,22 @@ def swap_dictionary_list_axis(alist):
     return result
 
 
+class IteratorPlusLength:
+
+    def __init__(self, generator, length):
+        self.length = length
+        self.generator = generator
+
+    def next(self):
+        return self.generator.next()
+
+    def __len__(self):
+        return self.length
+
+    def __iter__(self):
+        return self
+
+
 try:
     import rpy2.robjects as ro
     import rpy2.rinterface as rinterface
@@ -1327,12 +1430,15 @@ try:
             # Array-of-PyObject is treated like a Python list:
             elif o.dtype.kind == "O":
                 all_str = True
+                all_bool = True
                 for value in o:
-                    if not type(value) is str and not type(value) is unicode:
+                    if not type(value) is str and not type(value) is unicode and not type(value) is numpy.string_ and not (type(value) is numpy.ma.core.MaskedArray and value.mask == True):
                         all_str = False
                         break
-                if not all_str:
-                    raise(ValueError("numpy2ri_vector currently does not handle object vectors"))
+                    if not type(value) is bool or type(value) is numpy.bool_:
+                        all_bool = False
+                if (not all_str) and (not all_bool):
+                    raise(ValueError("numpy2ri_vector currently does not handle object vectors: %s %s" % (value, type(value))))
                 else:
                     #since we keep strings as objects
                     #we have to jump some hoops here
@@ -1348,22 +1454,37 @@ try:
         else:
             raise(ValueError("Unknown input to numpy2ri_vector."))
         return res
-   
+
+    ro.r("""
+        dataframe_colname_setting = function(df, names)
+        {
+            colnames(df) = names
+            df
+        }
+        """)
+       
     def dataframe2ri(o):
         #print 'converting', o, type(o)
         if isinstance(o, DataFrame):
             #print 'dataframe'
             dfConstructor = ro.r['data.frame']
-            parameters = {}
-            for column_name in o.value_dict:
+            names = []
+            parameters = []
+            kw_params = {}
+            for column_name in o.columns_ordered:
                 col = o.value_dict[column_name]
-                parameters[str(column_name)] = numpy2ri_vector(col)
+                try:
+                    names.append(str(column_name))
+                    parameters.append(numpy2ri_vector(col))
+                except ValueError, e:
+                    raise ValueError(str(e) + ' Offending column: %s, dtype' % column_name, col.dtype)
             if not o.row_names is None:
-                parameters['row.names'] = numpy2ri_vector(o.row_names)
+                kw_params['row.names'] = numpy2ri_vector(o.row_names)
             #print parameters
             ro.conversion.py2ri = previous_py2ri
             try:
-                res = dfConstructor(**parameters)
+                res = dfConstructor(*parameters, **kw_params)
+                res = ro.r('dataframe_colname_setting')(res, names)
             except TypeError:
                 print parameters.keys()
                 raise
