@@ -101,14 +101,17 @@ class DF2CSV:
             filehandle = _open_file(filename_or_file_object,'rU') 
             return self.__encapsulated_read(filehandle,
                                            header, dialect, skip_rows, columns_to_include, columns_to_execlude, na_text, type_hints, handle_quotes, has_row_names, r_skipped_first_column_name, comment_character)
-        except zlib.error:
-            filehandle = _open_file(filename_or_file_object,'r') 
-            return self.__encapsulated_read(filehandle,
-                                           header, dialect, skip_rows, columns_to_include, columns_to_execlude, na_text, type_hints, handle_quotes, has_row_names, r_skipped_first_column_name, comment_character)
+        except (zlib.error, IOError), e:
+            if isinstance(e, zlib.error) or str(e).find('CRC check failed') != -1:
+                filehandle = _open_file(filename_or_file_object,'r') 
+                return self.__encapsulated_read(filehandle,
+                                               header, dialect, skip_rows, columns_to_include, columns_to_execlude, na_text, type_hints, handle_quotes, has_row_names, r_skipped_first_column_name, comment_character)
+            else:
+                raise e
 
     def __encapsulated_read(self, filehandle, header=True, dialect=None, skip_rows = 0, columns_to_include = None, columns_to_execlude = None, na_text = "NA", type_hints = None, handle_quotes = False, has_row_names = False, r_skipped_first_column_name = False, comment_character = None):
         if dialect is None:
-            dialect = 'excel'
+            dialect = ExcelDialect()
         if isinstance(dialect, str):
             dialect = csv.get_dialect(dialect)
 
@@ -211,9 +214,17 @@ class DF2CSV:
                 del fields_used[0]
         return DataFrame( results, columns_ordered=fields_used, row_names_ordered = row_names )
 
+    def _truncate_if_necessary(self, value, dialect):
+        if dialect.max_entry_size:
+            if isinstance(value, str) and len(value) > dialect.max_entry_size:
+                return value[:dialect.max_entry_size] + '...'
+        return value
+    
     def write(self, data_frame, filename_or_file_object, dialect='excel'):
         """Write a DataFrame to a comma seperated value file."""
         filehandle = _open_file(filename_or_file_object, 'wb')
+        if dialect == 'excel':
+            dialect = ExcelDialect()
         writer = csv.writer(filehandle, dialect=dialect)
         if not data_frame.row_names is None:
             header = ["Row"]
@@ -225,12 +236,13 @@ class DF2CSV:
             rowNames = data_frame.row_names
             for i in range(data_frame.num_rows):
                 row = [rowNames[i]]
-                row.extend(data_frame.get_row_as_list(i))
+                row.extend(self._truncate_if_necessary(x, dialect) for x in data_frame.get_row_as_list(i))
                 writer.writerow(row)
         else:
             for i in range(data_frame.num_rows):
-                writer.writerow(data_frame.get_row_as_list(i))
-        filehandle.close()
+                writer.writerow([self._truncate_if_necessary(x, dialect) for x in data_frame.get_row_as_list(i)])
+        if not hasattr(filehandle, 'getvalue'): #this should cover both stringio and cStringIO - the later doesn't export a class, apparently.
+            filehandle.close()
 
 
     def _csvToColumns(self, file_object, dialect, number_of_columns, na_text, comment_character):
@@ -313,7 +325,7 @@ class DF2ARFF:
 class DF2Excel:
     """Use xlrd and xlwt to write 'real' excel files"""
 
-    def read(self, filename, sheet_name = None, row_offset = 0, col_offset = 0, row_max = None, col_max = None, filter_empty = False, NA_string = None, handle_encoding_errors = lambda x: str(x).strip()):
+    def read(self, filename, sheet_name = None, row_offset = 0, col_offset = 0, row_max = None, col_max = None, filter_empty = False, NA_string = None, handle_encoding_errors = lambda u: u.encode('latin-1', 'replace').strip().decode('latin-1')):
         import xlrd
         wb = xlrd.open_workbook(filename)
         if sheet_name is None:
@@ -349,7 +361,7 @@ class DF2Excel:
                     if type(value) == unicode and value == NA_string:
                         value = None
                 except UnicodeEncodeError:
-                    handle_encoding_errors(value)
+                    value = handle_encoding_errors(value)
                 cols[col_names_in_order[y - col_offset]].append((value))
                 if value:
                     found = True
@@ -409,15 +421,19 @@ class DF2Excel:
             if not value:
                 return ""
             else:
-                return unicode(value)
+                return unicode(value, 'latin-1', 'replace')[:65000]
         elif typ is float:
             if numpy.isnan(value):
                 return None
+            elif numpy.isinf(value):
+                return "INF"
             else:
                 return value
         elif typ is numpy.float64:
             if numpy.isnan(value):
                 return None
+            elif numpy.isinf(value):
+                return "INF"
             else:
                 return float(value)
         elif typ is int:
@@ -434,7 +450,7 @@ class DF2Excel:
             try:
                 return float(value)
             except:
-                return unicode(value)
+                return unicode(value)[:65000]
 
 class DF2Sqlite:
 
@@ -450,7 +466,7 @@ class DF2Sqlite:
             #print self._create_table_statement(table_name, data_frame)
             cur.execute(self._create_table_statement(table_name, data_frame))
             ins_statement = "INSERT INTO '%s' VALUES (%s)" % (table_name, ",".join(["?"] * len(data_frame.columns_ordered)))
-            cur.executemany(ins_statement, data_frame.iter_rows_list())
+            cur.executemany(ins_statement, data_frame.iter_rows_as_list())
         conn.commit()
         conn.close()
 
@@ -484,6 +500,31 @@ class DF2Sqlite:
         statement = 'CREATE TABLE \'%s\' (%s)' % (table_name, ",\n".join(field_defs)) 
         return statement
 
+class DF2HTMLTable:
+
+    def read(self, extracted_table):
+        """A terribly simple table parser"""
+        from BeautifulSoup import BeautifulSoup
+        soup = BeautifulSoup(extracted_table)
+        table = soup.table
+        rows = table.findAll('tr')
+        col_names = [] 
+        for td in rows[0].findAll('td'):
+            col_names.append(str(td.string))
+        data = {}
+        for name in col_names:
+            data[name] = []
+        for row in rows[1:]:
+            for ii, td in enumerate(row.findAll('td')):
+                if ii == len(col_names):
+                    break
+                data[col_names[ii]].append(str(td.string))
+            for ii in xrange(ii + 1, len(col_names)):
+                data[col_names[ii]].append('')
+        return DataFrame(data)
+
+
+
 
 
 
@@ -508,6 +549,7 @@ class TabDialect(csv.Dialect):
     quoting = csv.QUOTE_MINIMAL
     lineterminator = '\n'
     skipinitialspace = True
+    max_entry_size = 65000 #entries are truncated during write and '...' is appended.
 
 class CommaDialect(csv.Dialect):
     """A dialect to interpret tab separated files.
@@ -518,6 +560,11 @@ class CommaDialect(csv.Dialect):
     quoting = csv.QUOTE_MINIMAL
     lineterminator = '\n'
     skipinitialspace = True
+    max_entry_size = 65000
+    
+    def __init__(self, **args):
+        for kw in args:
+            setattr(self, kw, args[kw])
 
 
 class SpaceDialect(csv.Dialect):
@@ -529,4 +576,14 @@ class SpaceDialect(csv.Dialect):
     quoting = csv.QUOTE_MINIMAL
     lineterminator = '\n'
     skipinitialspace = True
+    max_entry_size = None
+
+class ExcelDialect(csv.Dialect):
+    delimiter = ';'
+    quotechar = '"'
+    doublequote = True
+    quoting = csv.QUOTE_MINIMAL
+    lineterminator = '\n'
+    skipinitialspace = True
+    max_entry_size = 65000
 
