@@ -346,6 +346,23 @@ class DataFrameTests(unittest.TestCase):
             {"Field2": 'hgi', 'source': 'Field3', 'value': 30}])
         self.assertEqual(x,supposed_melted)
 
+    def test_types(self):
+        a = DataFrame( { "Field1": [1, 2,3], "Field2": ['shu','def','hgi'],"Field4": [3,'b','c'], 'Field3': [10.5,20,30],
+            'Field5': ['shu', unicode('sha'), u'rah']})
+        t = {}
+        for name, typ in a.types():
+            t[name] = typ
+        self.assertEqual(t['Field1'], numpy.int32)
+        self.assertEqual(t['Field2'], numpy.object)
+        self.assertEqual(type(a.get_column_view('Field2')[0]), str)
+        self.assertEqual(t['Field4'], numpy.object)
+        self.assertEqual(type(a.get_column_view('Field4')[0]), str)
+        self.assertEqual(t['Field3'], numpy.float)
+        self.assertEqual(t['Field5'], numpy.object)
+        self.assertEqual(type(a.get_column_view('Field5')[0]), unicode)
+
+
+
     def test_melt_two_id_columns(self):
         a = DataFrame( { "Field1": [1, 2,3], "Field2": ['shu','def','hgi'],"Field4": ['a','a','c'], 'Field3': [10,20,30]}, ['Field1','Field2'])
         x = a.melt(["Field2",'Field4'], ["Field1",'Field3'], 'source','value')
@@ -553,7 +570,18 @@ class DataFrameAccessorTests(unittest.TestCase):
             a = a.sort_by('FieldX')
         self.assertRaises(KeyError, inner)
     
+    def test_sort_by_invalid_field_raises2(self):
+        def inner():
+            a = DataFrame( { "Field1": [1, 2,3,4,5,6], "Field2": ['def','ab','jl','ab','hgi','pqr'], 'Field3': [7.0,8.0,9.0,10.0,11.0,12.0]}, ['Field1','Field2', 'Field3'])
+            a = a.sort_by('FieldX', True)
+        self.assertRaises(KeyError, inner)
 
+    def test_sort_by_invalid_field_raises3(self):
+        def inner():
+            a = DataFrame( { "Field1": [1, 2,3,4,5,6], "Field2": ['def','ab','jl','ab','hgi','pqr'], 'Field3': [7.0,8.0,9.0,10.0,11.0,12.0]}, ['Field1','Field2', 'Field3'])
+            a = a.sort_by(('Field1', 'FieldX'), (True, False))
+        self.assertRaises(KeyError, inner)
+  
     def testSortBySingleFieldReverse(self):
         a = DataFrame( { "Field1": [1, 2,3,4,5,6], "Field2": ['def','ab','jl','ab','hgi','pqr'], 'Field3': [7.0,8.0,9.0,10.0,11.0,12.0]}, ['Field1','Field2', 'Field3'])
         a = a.sort_by('Field1', False)
@@ -1162,6 +1190,12 @@ class DataFrameManipulationTests(unittest.TestCase):
         a.digitize_column('Field1', bins=[50,100])
         self.assertTrue((a.get_column('Field1') == [0,0,0,0,1,1,1,1]).all())
 
+    def test_single_rowed_dataframe_boolean_selection(self):
+        a = DataFrame({"Field1": [1]})
+        a[a[:,'Field1'] > 0,"Field1"] = 10
+        self.assertEqual(a.get_value(0,0), 10)
+
+
 class DataFrameAuxilaryTests(unittest.TestCase):
     def testConvertTo2Darray(self):
         a = DataFrame( { "Field1": [1, 2,3,4,5,6],  'Field3': [7.0,8.0,9.0,10.0,11.0,12.0]}, ['Field1', 'Field3'])
